@@ -11,6 +11,7 @@ import pkg_resources
 import six
 
 from rbtools.clients.errors import SCMError
+from rbtools.deprecation import RemovedInRBTools40Warning
 from rbtools.utils.process import execute
 
 
@@ -84,23 +85,109 @@ class SCMClient(object):
     These are used for fetching repository information and generating diffs.
     """
 
+    #: The name of the client.
+    #:
+    #: Type:
+    #:     unicode
     name = None
 
-    #: Whether or not the SCM client can generate a commit history.
+    #: A comma-separated list of SCMClient names on the server
+    #:
+    #: Version Added:
+    #:    3.0
+    #:
+    #: Type:
+    #:     unicode
+    server_tool_names = None
+
+    #: Whether the SCM uses server-side changesets
+    #:
+    #: Version Added:
+    #:     3.0
+    #:
+    #: Type:
+    #:     bool
+    supports_changesets = False
+
+    #: Whether the SCM client can generate a commit history.
+    #:
+    #: Type:
+    #:     bool
     supports_commit_history = False
+
+    #: Whether the SCM client's diff method takes the ``extra_args`` parameter.
+    #:
+    #: Type:
+    #:     bool
     supports_diff_extra_args = False
+
+    #: Whether the SCM client supports excluding files from the diff.
+    #:
+    #: Type:
+    #:     bool
     supports_diff_exclude_patterns = False
+
+    #: Whether the SCM client can generate diffs without renamed files.
+    #:
+    #: Type:
+    #:     bool
     supports_no_renames = False
+
+    #: Whether the SCM client supports generating parent diffs.
+    #:
+    #: Version Added:
+    #:     3.0
+    #:
+    #: Type:
+    #:     bool
+    supports_parent_diffs = False
+
+    #: Whether the SCM client supports reverting patches.
+    #:
+    #: Type:
+    #:     bool
     supports_patch_revert = False
 
+    #: Whether commits can be amended.
+    #:
+    #: Type:
+    #:     bool
     can_amend_commit = False
+
+    #: Whether the SCM can create merges.
+    #:
+    #: Type:
+    #:     bool
     can_merge = False
+
+    #: Whether commits can be pushed upstream.
+    #:
+    #: Type:
+    #:     bool
     can_push_upstream = False
+
+    #: Whether branch names can be deleted.
+    #:
+    #: Type:
+    #:     bool
     can_delete_branch = False
+
+    #: Whether new branches can be created.
+    #:
+    #: Type:
+    #:     bool
     can_branch = False
+
+    #: Whether new bookmarks can be created.
+    #:
+    #: Type:
+    #:     bool
     can_bookmark = False
 
     #: Whether commits can be squashed during merge.
+    #:
+    #: Type:
+    #:     bool
     can_squash_merges = False
 
     def __init__(self, config=None, options=None):
@@ -117,17 +204,86 @@ class SCMClient(object):
         self.options = options
         self.capabilities = None
 
+    def is_remote_only(self):
+        """Return whether this repository is operating in remote-only mode.
+
+        For some SCMs and some operations, it may be possible to operate
+        exclusively with a remote server and have no working directory.
+
+        Version Added:
+            3.0
+
+        Returns:
+            bool:
+            Whether this repository is operating in remote-only mode.
+        """
+        return False
+
+    def get_local_path(self):
+        """Return the local path to the working tree.
+
+        This is expected to be overridden by subclasses.
+
+        Version Added:
+            3.0
+
+        Returns:
+            unicode:
+            The filesystem path of the repository on the client system.
+        """
+        logging.warning('%s should implement a get_local_path method',
+                        self.__class__)
+        info = self.get_repository_info()
+        return info and info.local_path
+
     def get_repository_info(self):
         """Return repository information for the current working tree.
 
         This is expected to be overridden by subclasses.
 
+        Version Added:
+            3.0
+
         Returns:
             rbtools.clients.RepositoryInfo:
             The repository info structure.
         """
         return None
 
+    def find_matching_server_repository(self, repositories):
+        """Find a match for the repository on the server.
+
+        Version Added:
+            3.0
+
+        Args:
+            repositories (rbtools.api.resource.ListResource):
+                The fetched repositories.
+
+        Returns:
+            tuple:
+            A 2-tuple of :py:class:`~rbtools.api.resource.ItemResource`. The
+            first item is the matching repository, and the second is the
+            repository info resource.
+        """
+        return None, None
+
+    def get_repository_name(self):
+        """Return any repository name configured in the repository.
+
+        This is used as a fallback from the standard config options, for
+        repository types that support configuring the name in repository
+        metadata.
+
+        Version Added:
+            3.0
+
+        Returns:
+            unicode:
+            The configured repository name, or None.
+        """
+        return None
+
     def check_options(self):
         """Verify the command line options.
 
@@ -173,7 +329,7 @@ class SCMClient(object):
             unicode:
             The Review Board server URL, if available.
         """
-        return self._get_server_from_config(self.config, repository_info)
+        return None
 
     def parse_revision_spec(self, revisions=[]):
         """Parse the given revision spec.
@@ -237,8 +393,46 @@ class SCMClient(object):
             'tip': None,
         }
 
+    def get_tree_matches_review_request(self, review_request, revisions,
+                                        **kwargs):
+        """Return whether a review request matches revisions or tree state.
+
+        This works along with review request matching in tools like
+        :command:`rbt post` to match state in a review request (such as in
+        ``extra_data``) with the state in the local tree (such as the local
+        branch or SCM-specific identifiers other than a commit ID).
+
+        Subclasses can override this to implement their own matching logic.
+        By default, no additional logic is implemented.
+
+        Version Added:
+            3.1
+
+        Args:
+            review_request (rbtools.api.resource.ReviewRequestResource):
+                The review request being matched.
+
+            revisions (dict):
+                A dictionary of revisions, as returned by
+                :py:meth:`parse_revision_spec`.
+
+            **kwargs (dict, unused):
+                Additional keyword arguments, for future expansion.
+
+        Returns:
+            bool:
+            ``True`` if the review request is considered an exact match.
+
+            ``False`` if the review request should be explicitly discarded
+            as a possible match.
+
+            ``None`` if a match could not be determined based on available
+            information.
+        """
+        return None
+
     def diff(self, revisions, include_files=[], exclude_patterns=[],
-             no_renames=False, extra_args=[]):
+             no_renames=False, repository_info=None, extra_args=[]):
         """Perform a diff using the given revisions.
 
         This is expected to be overridden by subclasses.
@@ -255,32 +449,56 @@ class SCMClient(object):
                 A list of shell-style glob patterns to blacklist during diff
                 generation.
 
+            repository_info (rbtools.clients.RepositoryInfo, optional):
+                The repository info structure.
+
             extra_args (list, unused):
                 Additional arguments to be passed to the diff generation.
 
+            **kwargs (dict, unused):
+                Unused keyword arguments.
+
         Returns:
             dict:
-            A dictionary containing the following keys:
+            A dictionary containing:
 
-            ``diff`` (:py:class:`bytes`):
-                The contents of the diff to upload.
+            Keys:
+                diff (bytes):
+                    The contents of the diff to upload.
 
-            ``parent_diff`` (:py:class:`bytes`, optional):
-                The contents of the parent diff, if available.
+                parent_diff (bytes, optional):
+                    The contents of the parent diff, if available.
 
-            ``commit_id`` (:py:class:`unicode`, optional):
-                The commit ID to include when posting, if available.
+                commit_id (unicode, optional):
+                    The commit ID to include when posting, if available.
 
-            ``base_commit_id` (:py:class:`unicode`, optional):
-                The ID of the commit that the change is based on, if available.
-                This is necessary for some hosting services that don't provide
-                individual file access.
+                base_commit_id (unicode, optional):
+                    The ID of the commit that the change is based on, if
+                    available.  This is necessary for some hosting services
+                    that don't provide individual file access.
+
+                review_request_extra_data (dict, optional):
+                    A dictionary of ``extra_data`` keys to set on the review
+                    request (when posting to Review Board 3.0 or higher).
+
+                    If posting a brand-new review request, this will set the
+                    fields on the review request itself.
+
+                    If updating a review request, this will set them on the
+                    draft.
+
+                    This may contain structured data. It will be sent to the
+                    server as part of a JSON Merge Patch.
+
+                    Version Added:
+                        3.1
         """
         return {
             'diff': None,
             'parent_diff': None,
             'commit_id': None,
             'base_commit_id': None,
+            'review_request_extra_data': None,
         }
 
     def get_commit_history(self, revisions):
@@ -299,44 +517,6 @@ class SCMClient(object):
         """
         raise NotImplementedError
 
-    def _get_server_from_config(self, config, repository_info):
-        """Return the Review Board server URL in the config.
-
-        Args:
-            config (dict):
-                The loaded user config.
-
-            repository_info (rbtools.clients.RepositoryInfo):
-                The repository info structure.
-
-        Returns:
-            unicode:
-            The server URL, if available.
-        """
-        if 'REVIEWBOARD_URL' in config:
-            return config['REVIEWBOARD_URL']
-        elif 'TREES' in config:
-            trees = config['TREES']
-            if not isinstance(trees, dict):
-                raise ValueError('"TREES" in config file is not a dict!')
-
-            # If repository_info is a list, check if any one entry is in trees.
-            path = None
-
-            if isinstance(repository_info.path, list):
-                for path in repository_info.path:
-                    if path in trees:
-                        break
-                else:
-                    path = None
-            elif repository_info.path in trees:
-                path = repository_info.path
-
-            if path and 'REVIEWBOARD_URL' in trees[path]:
-                return trees[path]['REVIEWBOARD_URL']
-
-        return None
-
     def _get_p_number(self, base_path, base_dir):
         """Return the appropriate value for the -p argument to patch.
 
@@ -436,13 +616,14 @@ class SCMClient(object):
             p_num = int(p_num)
         except ValueError:
             p_num = 0
-            logging.warn('Invalid -p value: %s; assuming zero.', p_num)
+            logging.warning('Invalid -p value: %s; assuming zero.', p_num)
 
         if p_num is not None:
             if p_num >= 0:
                 cmd.append('-p%d' % p_num)
             else:
-                logging.warn('Unsupported -p value: %d; assuming zero.', p_num)
+                logging.warning('Unsupported -p value: %d; assuming zero.',
+                                p_num)
 
         cmd.extend(['-i', six.text_type(patch_file)])
 
@@ -707,6 +888,17 @@ class RepositoryInfo(object):
                  supports_changesets=False, supports_parent_diffs=False):
         """Initialize the object.
 
+        Version Changed:
+            3.0:
+            The ``name`` argument was deprecated. Clients which allow
+            configuring the repository name in metadata should instead
+            implement :py:meth:`get_repository_name`.
+
+            The ``supports_changesets`` and ``supports_parent_diffs`` arguments
+            were deprecated. Clients which need these should instead set
+            :py:attr:`supports_changesets` and :py:attr:`supports_parent_diffs`
+            on themselves.
+
         Args:
             path (unicode or list of unicode, optional):
                 The path of the repository, or a list of possible paths
@@ -736,11 +928,36 @@ class RepositoryInfo(object):
         self.path = path
         self.base_path = base_path
         self.local_path = local_path
-        self.name = name
         self.supports_changesets = supports_changesets
         self.supports_parent_diffs = supports_parent_diffs
         logging.debug('Repository info: %s', self)
 
+        if name is not None:
+            RemovedInRBTools40Warning.warn(
+                'The name argument to RepositoryInfo has been deprecated and '
+                'will be removed in RBTools 4.0. Implement '
+                'get_repository_name instead.')
+
+        if supports_changesets:
+            # We can't make this a soft deprecation so raise an error instead.
+            # Users with custom SCMClient implementations will need to update
+            # their code when moving to RBTools 3.0+.
+            raise Exception(
+                'The supports_changesets argument to RepositoryInfo has been '
+                'deprecated and will be removed in RBTools 4.0. Clients which '
+                'rely on this must instead set the supports_changesets '
+                'attribute on the class.')
+
+        if supports_parent_diffs:
+            # We can't make this a soft deprecation so raise an error instead.
+            # Users with custom SCMClient implementations will need to update
+            # their code when moving to RBTools 3.0+.
+            raise Exception(
+                'The supports_changesets argument to RepositoryInfo has been '
+                'deprecated and will be removed in RBTools 4.0. Clients which '
+                'rely on this must instead set the supports_parent_diffs '
+                'attribute on the class.')
+
     def __str__(self):
         """Return a string representation of the repository info.
 
@@ -748,8 +965,7 @@ class RepositoryInfo(object):
             unicode:
             A loggable representation.
         """
-        return 'Path: %s, Base path: %s, Supports changesets: %s' % \
-            (self.path, self.base_path, self.supports_changesets)
+        return 'Path: %s, Base path: %s' % (self.path, self.base_path)
 
     def set_base_path(self, base_path):
         """Set the base path of the repository info.
@@ -766,12 +982,32 @@ class RepositoryInfo(object):
                       self.base_path, base_path)
         self.base_path = base_path
 
+    def update_from_remote(self, repository, info):
+        """Update the info from a remote repository.
+
+        Subclasses may override this to fetch additional data from the server.
+
+        Args:
+            repository (rbtools.api.resource.ItemResource):
+                The repository resource.
+
+            info (rbtools.api.resource.ItemResource):
+                The repository info resource.
+        """
+        self.path = repository.path
+
     def find_server_repository_info(self, server):
         """Try to find the repository from the list of repositories on the server.
 
         For Subversion, this could be a repository with a different URL. For
         all other clients, this is a noop.
 
+        Deprecated:
+            3.0:
+            Commands which need to use the remote repository, or need data from
+            the remote repository such as the base path, should set
+            :py:attr:`needs_repository`.
+
         Args:
             server (rbtools.api.resource.RootResource):
                 The root resource for the Review Board server.
@@ -780,6 +1016,11 @@ class RepositoryInfo(object):
             RepositoryInfo:
             The server-side information for this repository.
         """
+        RemovedInRBTools40Warning.warn(
+            'The find_server_repository_info method is deprecated, and will '
+            'be removed in RBTools 4.0. If you need to access the remote '
+            'repository, set the needs_repository attribute on your Command '
+            'subclass.')
         return self
 
 
@@ -806,8 +1047,7 @@ def load_scmclients(config, options):
             logging.exception('Could not load SCM Client "%s"', ep.name)
 
 
-def scan_usable_client(config, options, client_name=None,
-                       require_repository_info=True):
+def scan_usable_client(config, options, client_name=None):
     """Scan for a usable SCMClient.
 
     Args:
@@ -822,18 +1062,11 @@ def scan_usable_client(config, options, client_name=None,
             can be used to disambiguate if there are nested repositories, or to
             speed up detection.
 
-        require_repository_info (bool, optional):
-            Whether information on a repository is required. This is the
-            default. If disabled, this will return ``None`` for the repository
-            information if a matching repository could not be found.
-
     Returns:
         tuple:
         A 2-tuple, containing the repository info structure and the tool
         instance.
     """
-    from rbtools.clients.perforce import PerforceClient
-
     repository_info = None
     tool = None
 
@@ -854,69 +1087,60 @@ def scan_usable_client(config, options, client_name=None,
     else:
         scmclients = SCMCLIENTS
 
-    candidate_repos = []
-
+    # First go through and see if any repositories are configured in
+    # remote-only mode. For example, SVN can post changes purely with a remote
+    # URL and no working directory.
     for name, tool in six.iteritems(scmclients):
-        logging.debug('Checking for a %s repository...', tool.name)
-        repository_info = tool.get_repository_info()
+        if tool.is_remote_only():
+            break
+    else:
+        tool = None
 
-        if repository_info:
-            candidate_repos.append((repository_info, tool))
+    # Now scan through the repositories to find any local working directories.
+    # If there are multiple repositories which appear to be active in the CWD,
+    # choose the deepest and emit a warning.
+    if tool is None:
+        candidate_repos = []
+
+        for name, tool in six.iteritems(scmclients):
+            logging.debug('Checking for a %s repository...', tool.name)
+            local_path = tool.get_local_path()
+
+            if local_path:
+                candidate_repos.append((local_path, tool))
 
-    if candidate_repos:
         if len(candidate_repos) == 1:
-            repository_info, tool = candidate_repos[0]
-        else:
+            tool = candidate_repos[0][1]
+        elif candidate_repos:
             logging.debug('Finding deepest repository of multiple matching '
                           'repository types.')
 
             deepest_repo_len = 0
-            deepest_repo_info = None
             deepest_repo_tool = None
+            deepest_local_path = None
+            found_multiple = False
 
-            for repo, tool in candidate_repos:
-                if (repo.local_path and
-                    len(os.path.normpath(repo.local_path)) > deepest_repo_len):
-                    deepest_repo_len = len(repo.local_path)
-                    deepest_repo_info = repo
+            for local_path, tool in candidate_repos:
+                if len(os.path.normpath(local_path)) > deepest_repo_len:
+                    if deepest_repo_tool is not None:
+                        found_multiple = True
+
+                    deepest_repo_len = len(local_path)
                     deepest_repo_tool = tool
+                    deepest_local_path = local_path
 
-            if deepest_repo_info:
-                repository_info = deepest_repo_info
-                tool = deepest_repo_tool
+            if found_multiple:
+                logging.warning('Multiple matching repositories were found. '
+                                'Using %s repository at %s.',
+                                tool.name, deepest_local_path)
+                logging.warning('Define REPOSITORY_TYPE in .reviewboardrc if '
+                                'you wish to use a different repository.')
 
-                logging.warn('Multiple matching repositories were found. '
-                             'Using %s repository at %s.',
-                             tool.name, repository_info.local_path)
-                logging.warn('Define REPOSITORY_TYPE in .reviewboardrc if '
-                             'you wish to use a different repository.')
-            else:
-                # If finding the deepest repository fails (for example, when
-                # posting against a remote SVN repository there will be no
-                # local path), just default to the first repository found
-                repository_info, tool = candidate_repos[0]
+            tool = deepest_repo_tool
 
-    if repository_info is not None:
-        # Verify that options specific to an SCM Client have not been mis-used.
-        if (getattr(options, 'change_only', False) and
-            not repository_info.supports_changesets):
-            logging.error('The --change-only option is not valid for the '
-                          'current SCM client.\n')
-            sys.exit(1)
+    repository_info = tool and tool.get_repository_info()
 
-        if (getattr(options, 'parent_branch', None) and
-            not repository_info.supports_parent_diffs):
-            logging.error('The --parent option is not valid for the '
-                          'current SCM client.')
-            sys.exit(1)
-
-        if (not isinstance(tool, PerforceClient) and
-            (getattr(options, 'p4_client', None) or
-             getattr(options, 'p4_port', None))):
-            logging.error('The --p4-client and --p4-port options are not '
-                          'valid for the current SCM client.\n')
-            sys.exit(1)
-    elif require_repository_info:
+    if repository_info is None:
         if client_name:
             logging.error('The provided repository type was not detected '
                           'in the current directory.')
@@ -929,36 +1153,26 @@ def scan_usable_client(config, options, client_name=None,
 
         sys.exit(1)
 
+    # Verify that options specific to an SCM Client have not been mis-used.
+    if (getattr(options, 'change_only', False) and
+        not tool.supports_changesets):
+        logging.error('The --change-only option is not valid for the '
+                      'current SCM client.\n')
+        sys.exit(1)
+
+    if (getattr(options, 'parent_branch', None) and
+        not tool.supports_parent_diffs):
+        logging.error('The --parent option is not valid for the '
+                      'current SCM client.')
+        sys.exit(1)
+
+    from rbtools.clients.perforce import PerforceClient
+
+    if (not isinstance(tool, PerforceClient) and
+        (getattr(options, 'p4_client', None) or
+         getattr(options, 'p4_port', None))):
+        logging.error('The --p4-client and --p4-port options are not '
+                      'valid for the current SCM client.\n')
+        sys.exit(1)
+
     return repository_info, tool
-
-
-def print_clients(config, options):
-    """Print the supported detected SCM clients.
-
-    Each SCM client, including those provided by third party packages,
-    will be printed. Additionally, SCM clients which are detected in
-    the current directory will be highlighted.
-
-    Args:
-        config (dict):
-            The loaded user config.
-
-        options (argparse.Namespace):
-            The parsed command line options.
-    """
-    print('The following repository types are supported by this installation')
-    print('of RBTools. Each "<type>" may be used as a value for the')
-    print('"--repository-type=<type>" command line argument. Repository types')
-    print('which are detected in the current directory are marked with a "*"')
-    print('[*] "<type>": <Name>')
-
-    if SCMCLIENTS is None:
-        load_scmclients(config, options)
-
-    for name, tool in six.iteritems(SCMCLIENTS):
-        repository_info = tool.get_repository_info()
-
-        if repository_info:
-            print(' * "%s": %s' % (name, tool.name))
-        else:
-            print('   "%s": %s' % (name, tool.name))
