@@ -23,6 +23,8 @@ class PlasticClient(SCMClient):
     """
 
     name = 'Plastic'
+    server_tool_names = 'Plastic SCM'
+    supports_changesets = True
     supports_patch_revert = True
 
     REVISION_CHANGESET_PREFIX = 'cs:'
@@ -36,12 +38,12 @@ class PlasticClient(SCMClient):
         """
         super(PlasticClient, self).__init__(**kwargs)
 
-    def get_repository_info(self):
-        """Return repository information for the current working tree.
+    def get_local_path(self):
+        """Return the local path to the working tree.
 
         Returns:
-            rbtools.clients.RepositoryInfo:
-            The repository info structure.
+            unicode:
+            The filesystem path of the repository on the client system.
         """
         if not check_install(['cm', 'version']):
             logging.debug('Unable to execute "cm version": skipping Plastic')
@@ -66,12 +68,22 @@ class PlasticClient(SCMClient):
         if not m:
             return None
 
-        path = m.group(1)
+        return m.group(1)
 
-        return RepositoryInfo(path=path,
-                              local_path=path,
-                              supports_changesets=True,
-                              supports_parent_diffs=False)
+    def get_repository_info(self):
+        """Return repository information for the current working tree.
+
+        Returns:
+            rbtools.clients.RepositoryInfo:
+            The repository info structure.
+        """
+        local_path = self.get_local_path()
+
+        if local_path:
+            return RepositoryInfo(path=local_path,
+                                  local_path=local_path)
+
+        return None
 
     def parse_revision_spec(self, revisions=[]):
         """Parse the given revision spec.
@@ -120,7 +132,7 @@ class PlasticClient(SCMClient):
             raise TooManyRevisionsError
 
     def diff(self, revisions, include_files=[], exclude_patterns=[],
-             no_renames=False, extra_args=[]):
+             extra_args=[], **kwargs):
         """Perform a diff across all modified files in a Plastic workspace.
 
         Parent diffs are not supported (the second value in the tuple).
@@ -140,6 +152,9 @@ class PlasticClient(SCMClient):
             extra_args (list, unused):
                 Additional arguments to be passed to the diff generation.
 
+            **kwargs (dict, unused):
+                Unused keyword arguments.
+
         Returns:
             dict:
             A dictionary containing the following keys:
