@@ -7,28 +7,29 @@ import re
 import shutil
 import tempfile
 import time
+import unittest
 from hashlib import md5
 from random import randint
 from textwrap import dedent
 
 from kgb import SpyAgency
-from nose import SkipTest
 from six.moves import range
 
 from rbtools.clients import RepositoryInfo
 from rbtools.clients.errors import CreateCommitError, MergeError
 from rbtools.clients.mercurial import MercurialClient, MercurialRefType
 from rbtools.clients.tests import (FOO, FOO1, FOO2, FOO3, FOO4, FOO5, FOO6,
-                                   SCMClientTests)
+                                   SCMClientTestCase)
 from rbtools.utils.encoding import force_unicode
-from rbtools.utils.filesystem import is_exe_in_path, load_config, make_tempdir
+from rbtools.utils.filesystem import is_exe_in_path, load_config
 from rbtools.utils.process import execute
 
 
-class MercurialTestBase(SCMClientTests):
+class MercurialTestCase(SCMClientTestCase):
     """Base class for all Mercurial unit tests."""
 
-    def run_hg(self, command, **kwargs):
+    @classmethod
+    def run_hg(cls, command, **kwargs):
         """Run a Mercurial command.
 
         Args:
@@ -50,6 +51,7 @@ class MercurialTestBase(SCMClientTests):
             },
             split_lines=False,
             results_unicode=False,
+            with_errors=False,
             **kwargs)
 
     def hg_add_file_commit(self, filename='test.txt', data=b'Test',
@@ -93,7 +95,7 @@ class MercurialTestBase(SCMClientTests):
             self.run_hg(['tag', tag])
 
 
-class MercurialClientTests(SpyAgency, MercurialTestBase):
+class MercurialClientTests(SpyAgency, MercurialTestCase):
     """Unit tests for MercurialClient."""
 
     TESTSERVER = 'http://127.0.0.1:8080'
@@ -120,18 +122,38 @@ class MercurialClientTests(SpyAgency, MercurialTestBase):
             'email': 'email',
         })
 
+    @classmethod
+    def setup_checkout(cls, checkout_dir):
+        """Populate a Mercurial clone.
+
+        This will create a clone of the sample Mercurial repository stored in
+        the :file:`testdata` directory.
+
+        Args:
+            checkout_dir (unicode):
+                The top-level directory in which the clone will be placed.
+
+        Returns:
+            The main clone directory, or ``None`` if :command:`hg` isn't
+            in the path.
+        """
+        if not is_exe_in_path('hg'):
+            return None
+
+        cls.hg_dir = os.path.join(cls.testdata_dir, 'hg-repo')
+        cls.run_hg(['clone', '--stream', cls.hg_dir, checkout_dir])
+
+        return checkout_dir
+
     def setUp(self):
+        if not is_exe_in_path('hg'):
+            raise unittest.SkipTest('hg not found in path')
+
         super(MercurialClientTests, self).setUp()
 
-        if not is_exe_in_path('hg'):
-            raise SkipTest('hg not found in path')
-
-        self.hg_dir = os.path.join(self.testdata_dir, 'hg-repo')
-        self.clone_dir = self.chdir_tmp()
+        self.clone_dir = self.checkout_dir
         self.clone_hgrc_path = os.path.join(self.clone_dir, '.hg', 'hgrc')
 
-        self.run_hg(['clone', '--stream', self.hg_dir, self.clone_dir])
-
         self.client = MercurialClient(options=self.options)
 
         with open(self.clone_hgrc_path, 'w') as fp:
@@ -156,8 +178,6 @@ class MercurialClientTests(SpyAgency, MercurialTestBase):
             hgpath = os.path.dirname(hgpath)
 
         self.assertEqual(self.hg_dir, hgpath)
-        self.assertTrue(ri.supports_parent_diffs)
-        self.assertFalse(ri.supports_changesets)
 
     def test_scan_for_server(self):
         """Testing MercurialClient.scan_for_server"""
@@ -1378,7 +1398,7 @@ class MercurialClientTests(SpyAgency, MercurialTestBase):
         return force_unicode(self.run_hg(['identify']).split()[0])
 
 
-class MercurialSubversionClientTests(MercurialTestBase):
+class MercurialSubversionClientTests(MercurialTestCase):
     """Unit tests for hgsubversion."""
 
     TESTSERVER = 'http://127.0.0.1:8080'
@@ -1481,7 +1501,7 @@ class MercurialSubversionClientTests(MercurialTestBase):
         super(MercurialSubversionClientTests, self).setUp()
 
         if self._skip_reason:
-            raise SkipTest(self._skip_reason)
+            raise unittest.SkipTest(self._skip_reason)
 
         home_dir = self.get_user_home()
         hgrc_path = os.path.join(home_dir, '.hgrc')
@@ -1501,15 +1521,16 @@ class MercurialSubversionClientTests(MercurialTestBase):
             self.clone_dir = os.path.join(home_dir, 'checkout.hg')
             self.run_hg(['clone', '--stream', self.svn_checkout_url,
                          self.clone_dir])
-        except (OSError, IOError) as e:
-            self.fail('Unable to clone Subversion repository: %s' % e)
+        except Exception as e:
+            raise unittest.SkipTest(
+                'Unable to clone Subversion repository: %s' % e)
 
         os.chdir(self.clone_dir)
         self.options.parent_branch = None
         self.client = MercurialClient(options=self.options)
 
     @classmethod
-    def svn_add_file_commit(self, filename, data, msg, add_file=True):
+    def svn_add_file_commit(cls, filename, data, msg, add_file=True):
         with open(filename, 'wb') as fp:
             fp.write(data)
 
