@@ -1,6 +1,4 @@
-from __future__ import print_function, unicode_literals
-
-import sys
+from __future__ import unicode_literals
 
 from rbtools.clients.errors import InvalidRevisionSpecError
 from rbtools.commands import Command, CommandError
@@ -13,6 +11,17 @@ class Diff(Command):
 
     name = 'diff'
     author = 'The Review Board Project'
+
+    # The diff command uses the API because some of the clients may change
+    # their diff behavior based on server capabilities (for example, very old
+    # servers may not have support for moved files with some SCM types). We
+    # might want to consider adding a mode for running this command in a purely
+    # offline way, or we might just want to define a supported baseline version
+    # of Review Board and get rid of some of the capability conditionals.
+    needs_api = True
+    needs_repository = True
+    needs_scm_client = True
+
     args = '[revisions]'
     option_list = [
         Command.server_options,
@@ -44,11 +53,7 @@ class Diff(Command):
                 'Subversion changelist, pass the changelist name as an '
                 'additional argument after the command.')
 
-        repository_info, tool = self.initialize_scm_tool(
-            client_name=self.options.repository_type)
-        server_url = self.get_server_url(repository_info, tool)
-        api_client, api_root = self.get_api(server_url)
-        self.setup_tool(tool, api_root=api_root)
+        tool = self.tool
 
         try:
             revisions = tool.parse_revision_spec(args)
@@ -85,6 +90,7 @@ class Diff(Command):
             revisions=revisions,
             include_files=self.options.include_files or [],
             exclude_patterns=self.options.exclude_patterns or [],
+            repository_info=self.repository_info,
             extra_args=extra_args,
             **diff_kwargs)
 
@@ -92,8 +98,8 @@ class Diff(Command):
 
         if diff:
             if six.PY2:
-                print(diff)
+                self.stdout.write(diff)
             else:
                 # Write the non-decoded binary diff to standard out
-                sys.stdout.buffer.write(diff)
-                print()
+                self.stdout_bytes.write(diff)
+                self.stdout.new_line()
