@@ -1,4 +1,4 @@
-from __future__ import print_function, unicode_literals
+from __future__ import unicode_literals
 
 import logging
 import re
@@ -10,17 +10,18 @@ except ImportError:
     from shutil import get_terminal_size
 
 from rbtools.commands import Command, Option
-from rbtools.utils.repository import get_repository_id
 from rbtools.utils.users import get_username
 
 
 class Status(Command):
     """Display review requests for the current repository."""
 
-
     name = 'status'
     author = 'The Review Board Project'
     description = 'Output a list of your pending review requests.'
+
+    needs_api = True
+
     args = '[review-request [revision]]'
     option_list = [
         Option('--format',
@@ -33,13 +34,14 @@ class Status(Command):
                     '`XX` the hex code of a character.\n'
                     'For example: --format="%%(id)s\\x09%%(summary)s"\n'
                     'This option will print out the ID and summary tab-'
-                    'separated.'),
+                    'separated. This is incompatible with --json.'),
         Option('-z',
                dest='format_nul',
                default=False,
                action='store_true',
                help='Null-terminate each entry. Otherwise, the entries will '
-                    'be newline-terminated.'),
+                    'be newline-terminated. This is incompatible with '
+                    '--json.'),
         Option('--all',
                dest='all_repositories',
                action='store_true',
@@ -51,9 +53,11 @@ class Status(Command):
         Command.perforce_options,
         Command.tfs_options,
     ]
+
     # The number of spaces between the request's status and the request's id
     # and summary.
     TAB_SIZE = 3
+
     # The number of spaces after the end of the request's summary.
     PADDING = 5
 
@@ -66,6 +70,8 @@ class Status(Command):
             review_requests (list of dict):
                 A list that contains statistics about each review request.
         """
+        self.json.add('review_requests', [])
+
         if len(review_requests):
             has_branches = False
             has_bookmarks = False
@@ -94,75 +100,104 @@ class Status(Command):
                     'r/%s - %s' % (info['id'], info['summary']),
                 ]
 
+                summary = {
+                    'approval_failure': info['approval_failure'],
+                    'approved': info['approved'],
+                    'description': info['description'],
+                    'has_draft': info['has_draft'],
+                    'open_issue_count': info['open_issue_count'],
+                    'review_request_id': info['id'],
+                    'review_request_url': info['url'],
+                    'shipit_count': info['shipit_count'],
+                    'status': info['status'],
+                    'summary': info['summary'],
+                }
+
                 if has_branches:
                     row.append(info.get('branch') or '')
+                    summary['branch'] = row[-1]
 
                 if has_bookmarks:
                     row.append(info.get('bookmark') or '')
+                    summary['bookmark'] = row[-1]
 
                 table.add_row(row)
+                self.json.append('review_requests', summary)
 
-            print(table.draw())
+            self.stdout.write(table.draw())
         else:
-            print('No review requests found.')
+            self.stdout.write('No review requests found.')
 
-        print()
+        self.stdout.new_line()
 
-    def get_data(self, requests):
-        """Return current status and review summary for all reviews.
+    def get_data(self, review_requests):
+        """Return current status and review summary for all review requests.
 
         Args:
-            requests (ListResource):
-                A ListResource that contains data on all open/draft requests.
+            review_requests (ListResource):
+                A ListResource that contains data on all open/draft
+                review requests.
 
         Returns:
-            list: A list whose elements are dicts of each request's statistics.
+            list:
+            A list whose elements are dicts of each review request's
+            statistics.
         """
-        requests_stats = []
+        review_requests_stats = []
 
-        for request in requests.all_items:
-            if request.draft:
+        for review_request in review_requests.all_items:
+            if review_request.draft:
                 status = 'Draft'
-            elif request.issue_open_count:
-                status = 'Open Issues (%s)' % request.issue_open_count
-            elif request.ship_it_count:
-                status = 'Ship It! (%s)' % request.ship_it_count
+            elif review_request.issue_open_count:
+                status = 'Open Issues (%s)' % review_request.issue_open_count
+            elif review_request.ship_it_count:
+                status = 'Ship It! (%s)' % review_request.ship_it_count
             else:
                 status = 'Pending'
 
-            if request.draft:
-                summary = request.draft[0]['summary']
+            if review_request.draft:
+                summary = review_request.draft[0]['summary']
             else:
-                summary = request.summary
+                summary = review_request.summary
 
             info = {
-                'id':  request.id,
+                'approval_failure': review_request.approval_failure,
+                'approved': review_request.approved,
+                'description': review_request.description,
+                'id':  review_request.id,
                 'status': status,
                 'summary': summary,
-                'description': request.description,
+                'url': review_request.absolute_url,
+                'shipit_count': review_request.ship_it_count,
+                'open_issue_count': review_request.issue_open_count,
+                'has_draft': review_request.draft is not None
             }
 
-            if 'local_branch' in request.extra_data:
-                info['branch'] = \
-                    request.extra_data['local_branch']
-            elif 'local_bookmark' in request.extra_data:
-                info['bookmark'] = \
-                    request.extra_data['local_bookmark']
+            if 'local_branch' in review_request.extra_data:
+                info['branch'] = review_request.extra_data['local_branch']
+            elif 'local_bookmark' in review_request.extra_data:
+                info['bookmark'] = review_request.extra_data['local_bookmark']
 
-            requests_stats.append(info)
+            review_requests_stats.append(info)
 
-        return requests_stats
+        return review_requests_stats
+
+    def initialize(self):
+        """Initialize the command.
+
+        This override of the base command initialize method exists so we can
+        conditionally set whether the SCM Client is necessary. Without any
+        options, this command will only print the status of review requests on
+        the current repository, which requires the client. If --all is passed,
+        this command only needs the API.
+        """
+        self.needs_repository = not self.options.all_repositories
+
+        super(Status, self).initialize()
 
     def main(self):
-        repository_info, tool = self.initialize_scm_tool(
-            client_name=self.options.repository_type)
-        server_url = self.get_server_url(repository_info, tool)
-        api_client, api_root = self.get_api(server_url)
-        self.setup_tool(tool, api_root=api_root)
-        username = get_username(api_client, api_root, auth_required=True)
-
-        # Check if repository info on reviewboard server match local ones.
-        repository_info = repository_info.find_server_repository_info(api_root)
+        username = get_username(self.api_client, self.api_root,
+                                auth_required=True)
 
         query_args = {
             'from_user': username,
@@ -171,20 +206,15 @@ class Status(Command):
         }
 
         if not self.options.all_repositories:
-            repo_id = get_repository_id(
-                repository_info,
-                api_root,
-                repository_name=self.options.repository_name)
-
-            if repo_id:
-                query_args['repository'] = repo_id
+            if self.repository:
+                query_args['repository'] = self.repository.id
             else:
                 logging.warning('The repository detected in the current '
                                 'directory was not found on\n'
                                 'the Review Board server. Displaying review '
                                 'requests from all repositories.')
 
-        review_requests = api_root.get_review_requests(**query_args)
+        review_requests = self.api_root.get_review_requests(**query_args)
         review_request_info = self.get_data(review_requests)
 
         if self.options.format:
@@ -210,4 +240,4 @@ class Status(Command):
             end = '\n'
 
         for info in review_requests:
-            print(fmt % info, end=end)
+            self.stdout.write(fmt % info, end=end)
